/*
 *
 *
 *    avraudio.h
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2016 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *
 *    Portions adapted from Instructables article by Amanda Ghassaei
 *      http://www.instructables.com/id/Arduino-Audio-Input/
 *      Sept 2012, Also under GPL3.
 *
 *
 */

#ifndef __AVR_AUDIO_H
#define __AVR_AUDIO_H

//
//
//    GetFs - calculate the sampling rate associated with a given divisor
//
//
inline uint32_t GetFs(uint32_t divisor) {
	return static_cast<uint32_t>(round(F_CPU / (13.0 * divisor)));
}

//
//  SetSamplingStatus(...) - enable or disable reading of A/D
//
inline void SetSamplingStatus(bool enabled) {
	// disable interrupts
	cli();

	if (enabled) {
		ADCSRA |= (1 << ADATE); // enable auto trigger
		ADCSRA |= (1 << ADIE);  // enable interrupt when measurement complete
		ADCSRA |= (1 << ADEN);  // enable ADC
		ADCSRA |= (1 << ADSC);  // start ADC measurements
	} else {
		ADCSRA &= ~(1 << ADATE); // disable auto trigger
		ADCSRA &= ~(1 << ADIE);  // disable A/D interrupt
		ADCSRA &= ~(1 << ADEN);  // disable ADC
		ADCSRA &= ~(1 << ADSC);  // stop ADC measurements
	}

	// enable interrupts
	sei();
}

//
//
//    ConfigureAnalogSampling - configure A0 to provide constant-rate data sampling
//
//
inline void ConfigureAnalogSampling(uint16_t divisor) {
	// disable interrupts
	cli();

	// clear ADCSRA and ADCSRB registers
	ADCSRA = 0;
	ADCSRB = 0;

	ADMUX |= (1 << REFS0); // set reference voltage
	ADMUX |= (1 << ADLAR); // left align the ADC value, so we can read highest 8 bits from ADCH register only

	uint8_t rateMask = 0;
	switch (divisor) {
		case 2: rateMask = 0b00000001; break;
		case 4: rateMask = 0b00000010; break;
		case 8: rateMask = 0b00000011; break;
		case 16: rateMask = 0b00000100; break;
		case 32: rateMask = 0b00000101; break;
		case 64: rateMask = 0b00000110; break;
		case 128: rateMask = 0b00000111; break;
	}
	ADCSRA &= 0b11111000;
	ADCSRA |= rateMask;

	// start sampling
	SetSamplingStatus(true);

	// enable interrupts
	sei();
}

#endif // __AVR_AUDIO_H
